#region Copyright Syncfusion Inc. 2001 - 2013
// Copyright Syncfusion Inc. 2001 - 2013. All rights reserved.
// Use of this code is subject to the terms of our license.
// A copy of the current license can be obtained at any time by e-mailing
// licensing@syncfusion.com. Any infringement will be prosecuted under
// applicable laws. 
#endregion

using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.Windows.Media;
using Syncfusion.UI.Xaml.Charts;
using Syncfusion.Windows.SampleLayout;
using System;
using System.Collections.ObjectModel;

namespace LineChart
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>       
    public partial class MainWindow : SampleLayoutWindow
    {
        readonly DataGenerator dataGenerator = new DataGenerator();
        private VerticalLineAnnotation verticalLine1;

        public MainWindow()
        {
            InitializeComponent();

            AddSerie(dataGenerator.DataSeries[0], "#FFFEB80A");
            AddSerie(dataGenerator.DataSeries[1], "#FF00FA9A");
            AddSerie(dataGenerator.DataSeries[2], "#FFFF1493");
            AddSerie(dataGenerator.DataSeries[3], "#FFFF4500");
            AddSerie(dataGenerator.DataSeries[4], "#FF00ADDC");
            
            this.lineChart.MouseRightButtonDown += ChartAreaMouseRightButtonDown;
        }

        private void AddSerie(DataSerie dataSerie, string color)
        {
            var cs = new LineSeries
                         {
                             Label = dataSerie.Name,
                             ItemsSource = dataSerie.Datas,
                             StrokeThickness=1,
                             XBindingPath = "Time",
                             Interior = new SolidColorBrush(StringToMediaColor(color)),
                             YBindingPath = "Value",
                             
                         };

            cs.AdornmentsInfo = new ChartAdornmentInfo()
                                    {
                                        Symbol = ChartSymbol.Ellipse,
                                        ShowMarker = true,
                                        SymbolHeight = 4,
                                        SymbolWidth = 4
                                    };

            cs.MouseMove += SeriesMouseMove;

            this.lineChart.Series.Add(cs);
            
        }

        

        void SeriesMouseMove(object sender, System.Windows.Input.MouseEventArgs e)
        {
            var point = e.GetPosition(this.lineChart);
            double xValue = this.lineChart.PointToValue(this.lineChart.PrimaryAxis, point);

            

            var serie = sender as LineSeries;
            if (serie == null) return;

            tbxSerieName.Text = serie.Label;
            tbxSerieName.Foreground = serie.Interior;

            var date = DateTime.FromOADate(xValue);
            tbxSelPointDate.Text = date.ToString("MMM-dd");



            //object selObj = DichotomySearch(serie., xValue);
            /*
            if (selObj != null)
            {
                SelectedPoint = selObj;
            }*/
        }
        /*
        private static object DichotomySearch(object chartData, double xValue)
        {
            int imin = 0;
            int imax = chartData.Count - 1;
            int median = (imax - imin) / 2;

            while (imax - imin > 1)
            {
                var dataPoint = chartData[median];
                if (dataPoint.X < xValue)
                {
                    imin = median;
                }
                else if (dataPoint.X > xValue)
                {
                    imax = median;
                }
                else
                {
                    return dataPoint.Item;
                }
                median = (imax - imin) / 2 + imin;
            }

            if (imax == imin)
                return chartData[imax].Item;


            var distToMin = Math.Abs(chartData[imin].X - xValue);
            var distToMax = Math.Abs(chartData[imax].X - xValue);

            return distToMin < distToMax
                       ? chartData[imin].Item
                       : chartData[imax].Item;

        }
        */
        public static Color StringToMediaColor(string strColor)
        {
            Byte a, r, g, b;
            Byte.TryParse(strColor.Substring(1, 2), NumberStyles.HexNumber, CultureInfo.InvariantCulture, out a);
            Byte.TryParse(strColor.Substring(3, 2), NumberStyles.HexNumber, CultureInfo.InvariantCulture, out r);
            Byte.TryParse(strColor.Substring(5, 2), NumberStyles.HexNumber, CultureInfo.InvariantCulture, out g);
            Byte.TryParse(strColor.Substring(7, 2), NumberStyles.HexNumber, CultureInfo.InvariantCulture, out b);
            return Color.FromArgb(a, r, g, b);
        }

        void ChartAreaMouseRightButtonDown(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {

            if (this.lineChart.Annotations.Count == 0)
            {
                verticalLine1 = new VerticalLineAnnotation
                {
                    Visibility = System.Windows.Visibility.Visible,
                    Cursor = System.Windows.Input.Cursors.ScrollWE,
                    CanDrag = true,
                    ShowAxisLabel = true,
                    XAxisName = "Time",
                    Stroke = new SolidColorBrush(StringToMediaColor("#FFF8821B")),
                    StrokeThickness = 2,
                    CoordinateUnit = CoordinateUnit.Axis,
                };

                var point = e.GetPosition(this.lineChart);
                double xValue = this.lineChart.PointToValue(this.lineChart.PrimaryAxis, point);
                this.lineChart.Annotations.Add(verticalLine1);
                
                verticalLine1.X1 = xValue;
                
            }
            else
            {
                this.lineChart.Annotations.Clear();
            }
            /*
            if (ChartArea.InteractiveCursors.Count == 0)
            {
                currentCursor = this.interactiveCursor1;
            }
            else if (ChartArea.InteractiveCursors.Count == 1)
            {
                currentCursor = this.interactiveCursor2;
            }
            else
            {
                if (ChartArea.InteractiveCursors.Count > 0)
                {
                    //foreach (var cursor in ChartArea.InteractiveCursors)
                    //    cursor.MouseUp -= InteractiveCursorMouseUp;

                    ChartArea.InteractiveCursors.Clear();
                }
                return;
            }

            if (currentCursor != null)
            {
                var point = e.GetPosition(ChartArea);
                currentCursor.OffsetX = point.X - (ChartArea.AxesThickness.Left + ChartArea.ElementMargin.Left * 4);
                currentCursor.IsBindWithMouseMove = true;

                ChartArea.InteractiveCursors.Add(currentCursor);

            }*/
        }

        

    }
   
    public class DataGenerator
    {
        private readonly List<DataSerie> dataSeries;

        public DataGenerator()
        {
            dataSeries = this.LoadData("..\\..\\Data\\histo1000_data.csv");
        }

        public List<DataSerie> DataSeries
        {
            get { return dataSeries; }
        }

        private int max;

        public int Max
        {
            get { return max; }
        }

        public List<DataSerie> LoadData(string fileName)
        {

            char[] comma = new char[] { ',' };
            char[] slashN = new char[] { '\n' };
            string s = File.ReadAllText(fileName);

            string[] lines = s.Split(slashN);

            int count = lines.Length;

            string[] values = lines[0].Split(comma);
            var vCount = values.Length;
            int serieCount = vCount / 2;

            var dataSeries = new List<DataSerie>(serieCount);
            for (int si = 0; si < serieCount; ++si)
            {
                dataSeries.Add(new DataSerie
                {
                    Name = values[si * 2 + 1].Trim(),
                    Datas = new ObservableCollection<Data>()
                });
            }

            for (int i = 1; i < count; ++i)
            {
                values = lines[i].Split(comma);
                if (values.Length != vCount)
                    continue;

                for (int si = 0; si < serieCount; ++si)
                {

                    var dateStr = values[si * 2];
                    var valuesStr = values[si * 2 + 1].Trim();

                    DateTime pointDateTime;

                    if (String.IsNullOrEmpty(dateStr) || !DateTime.TryParse(dateStr, out pointDateTime))
                    {
                        continue;
                    }
                    double pointValue;
                    if (String.IsNullOrEmpty(valuesStr) || !double.TryParse(valuesStr, out pointValue))
                    {
                        continue;
                    }

                    dataSeries[si].Datas.Add(new Data { Time = pointDateTime, Value = pointValue });
                }
            }

            max = 0;
            foreach (var serie in dataSeries)
            {
                max = Math.Max(max, serie.Datas.Count);
            }

            return dataSeries;
        }
    }

    public class DataSerie
    {
        public string Name { get; set; }
        public ObservableCollection<Data> Datas { get; set; }
    }

    public class Data
    {
        public DateTime Time { get; set; }
        public double Value { get; set; }
    }
}
